@@ -23,7 +23,7 @@ Animation and VFX Studios.
 See docs for more information.
 """
 
-__version__ = '0.2.11'
+__version__ = '0.2.12'
 
 
 import sys
@@ -40,8 +40,8 @@ from stalker.config import defaults
 
 from stalker.models.auth import Group, Permission, User, LocalSession, Role
 from stalker.models.asset import Asset
-from stalker.models.budget import Budget, BudgetEntry
-from stalker.models.client import Client
+from stalker.models.budget import Budget, BudgetEntry, Good, PriceList
+from stalker.models.client import Client, ClientUser
 from stalker.models.department import Department, DepartmentUser
 from stalker.models.entity import SimpleEntity, Entity
 from stalker.models.format import ImageFormat
@@ -50,9 +50,9 @@ from stalker.models.message import Message
 from stalker.models.mixins import (ProjectMixin, ReferenceMixin,
                                    DateRangeMixin, StatusMixin,
                                    TargetEntityTypeMixin, CodeMixin,
-                                   WorkingHoursMixin, ScheduleMixin)
+                                   WorkingHoursMixin, ScheduleMixin, DAGMixin)
 from stalker.models.note import Note
-from stalker.models.project import Project
+from stalker.models.project import Project, ProjectUser
 from stalker.models.review import Review, Daily, DailyLink
 from stalker.models.repository import Repository
 from stalker.models.scene import Scene
